@@ -334,8 +334,6 @@ gst_svtav1enc_class_init (GstSvtAv1EncClass * klass)
           "Colon-delimited list of key=value pairs of additional parameters to set",
           PROP_PARAMETERS_STRING_DEFAULT,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
-
-  gst_type_mark_as_plugin_api (GST_SVTAV1ENC_TYPE_INTRA_REFRESH_TYPE, 0);
 }
 
 static void
@@ -1123,12 +1121,6 @@ gst_svtav1enc_parse_parameters_string (GstSvtAv1Enc * svtav1enc)
   g_strfreev (key_values);
 }
 
-/**
- * plugin-svtav1:
- *
- * Since: 1.24
- */
-
 static gboolean
 plugin_init (GstPlugin * plugin)
 {
