@@ -38,7 +38,7 @@ const gchar *                       _sdp_source_to_string                   (SDP
 
 
 G_GNUC_INTERNAL
-gboolean                            validate_sdp                            (GstWebRTCBin * webrtc,
+gboolean                            validate_sdp                            (GstWebRTCSignalingState state,
                                                                              SDPSource source,
                                                                              GstWebRTCSessionDescription * sdp,
                                                                              GError ** error);
@@ -58,6 +58,8 @@ GstWebRTCRTPTransceiverDirection    _get_final_direction                     (Gs
 G_GNUC_INTERNAL
 GstWebRTCDTLSSetup                  _get_dtls_setup_from_media              (const GstSDPMedia * media);
 G_GNUC_INTERNAL
+GstWebRTCDTLSSetup                  _get_dtls_setup_from_session            (const GstSDPMessage * sdp);
+G_GNUC_INTERNAL
 GstWebRTCDTLSSetup                  _intersect_dtls_setup                   (GstWebRTCDTLSSetup offer);
 G_GNUC_INTERNAL
 void                                _media_replace_setup                    (GstSDPMedia * media,
@@ -76,5 +78,42 @@ G_GNUC_INTERNAL
 gboolean                            _media_has_attribute_key                (const GstSDPMedia * media,
                                                                              const gchar * key);
 
+G_GNUC_INTERNAL
+int                                 _get_sctp_port_from_media               (const GstSDPMedia * media);
+G_GNUC_INTERNAL
+guint64                             _get_sctp_max_message_size_from_media   (const GstSDPMedia * media);
+
+G_GNUC_INTERNAL
+void                                _get_ice_credentials_from_sdp_media     (const GstSDPMessage * sdp,
+                                                                             guint media_idx,
+                                                                             gchar ** ufrag,
+                                                                             gchar ** pwd);
+G_GNUC_INTERNAL
+gboolean                            _message_media_is_datachannel           (const GstSDPMessage * msg,
+                                                                             guint media_id);
+G_GNUC_INTERNAL
+guint                               _message_get_datachannel_index          (const GstSDPMessage * msg);
+G_GNUC_INTERNAL
+gboolean                            _message_has_attribute_key              (const GstSDPMessage * msg,
+                                                                             const gchar * key);
+
+G_GNUC_INTERNAL
+gboolean                            _get_bundle_index                       (GstSDPMessage * sdp,
+                                                                             GStrv bundled,
+                                                                             guint * idx);
+G_GNUC_INTERNAL
+gboolean                            _parse_bundle                           (GstSDPMessage * sdp,
+                                                                             GStrv * bundled,
+                                                                             GError ** error);
+
+G_GNUC_INTERNAL
+const gchar *                       _media_get_ice_pwd                  (const GstSDPMessage * msg,
+                                                                             guint media_idx);
+G_GNUC_INTERNAL
+const gchar *                       _media_get_ice_ufrag                (const GstSDPMessage * msg,
+                                                                             guint media_idx);
+
+G_GNUC_INTERNAL
+gboolean                            _media_is_bundle_only               (const GstSDPMedia * sdp);
 
 #endif /* __WEBRTC_UTILS_H__ */
