@@ -30,11 +30,5 @@
 #include <gst/analytics/gstanalyticsclassificationmtd.h>
 #include <gst/analytics/gstanalyticsobjectdetectionmtd.h>
 #include <gst/analytics/gstanalyticsobjecttrackingmtd.h>
-#include <gst/analytics/gstanalyticssegmentationmtd.h>
-#include <gst/analytics/gstanalyticstensormtd.h>
-#include <gst/analytics/gsttensormeta.h>
-#include <gst/analytics/gstanalyticsbatchmeta.h>
-#include <gst/analytics/gstanalytics_image_util.h>
-#include <gst/analytics/modelinfo.h>
 
 #endif /* __ANALYTICS_H__ */
