@@ -23,8 +23,9 @@
 #endif
 
 #include "gsth265bitwriter.h"
-#include "nalutils.h"
+#include <gst/codecparsers/nalutils.h>
 #include <gst/base/gstbitwriter.h>
+#include <math.h>
 
 #ifndef GST_DISABLE_GST_DEBUG
 #define GST_CAT_DEFAULT gst_h265_debug_category_get()
@@ -639,6 +640,7 @@ _h265_bit_writer_scaling_lists (const GstH265ScalingList * src_scaling_list,
       const guint8 *sl;
       const guint8 *default_sl;
       guint8 nextCoef;
+      gint8 coef_val;
       guint8 scaling_list_pred_matrix_id_delta;
 
       if (!_get_scaling_list_params (src_scaling_list, sizeId, matrixId,
@@ -659,7 +661,7 @@ _h265_bit_writer_scaling_lists (const GstH265ScalingList * src_scaling_list,
       /* If some previous matrix is the same, just ref it. */
       scaling_list_pred_matrix_id_delta = 0;
       for (j = 0; j < matrixId; j++) {
-        gboolean ret GST_UNUSED_ASSERT;
+        gboolean ret;
         guint8 size2;
         const guint8 *sl2;
         gint16 scaling_list_dc_coef_minus8_2 = 8;
@@ -695,7 +697,7 @@ _h265_bit_writer_scaling_lists (const GstH265ScalingList * src_scaling_list,
       }
 
       for (i = 0; i < size; i++) {
-        gint coef_val = sl[i] - nextCoef;
+        coef_val = sl[i] - nextCoef;
         nextCoef = sl[i];
 
         if (coef_val > 127) {
@@ -1564,8 +1566,10 @@ _h265_bit_writer_slice_header (const GstH265SliceHdr * slice,
     CtbLog2SizeY =
         MinCbLog2SizeY + sps->log2_diff_max_min_luma_coding_block_size;
     CtbSizeY = 1 << CtbLog2SizeY;
-    PicHeightInCtbsY = div_ceil (sps->pic_height_in_luma_samples, CtbSizeY);
-    PicWidthInCtbsY = div_ceil (sps->pic_width_in_luma_samples, CtbSizeY);
+    PicHeightInCtbsY =
+        ceil ((gdouble) sps->pic_height_in_luma_samples / (gdouble) CtbSizeY);
+    PicWidthInCtbsY =
+        ceil ((gdouble) sps->pic_width_in_luma_samples / (gdouble) CtbSizeY);
     PicSizeInCtbsY = PicWidthInCtbsY * PicHeightInCtbsY;
 
     n = gst_util_ceil_log2 (PicSizeInCtbsY);
@@ -2257,7 +2261,6 @@ gst_h265_bit_writer_convert_to_nal (guint nal_prefix_size,
       GST_H265_BIT_WRITER_ERROR);
   g_return_val_if_fail (raw_data != NULL, GST_H265_BIT_WRITER_ERROR);
   g_return_val_if_fail (raw_size > 0, GST_H265_BIT_WRITER_ERROR);
-  g_return_val_if_fail (raw_size / 8 <= G_MAXUINT, GST_H265_BIT_WRITER_ERROR);
   g_return_val_if_fail (nal_data != NULL, GST_H265_BIT_WRITER_ERROR);
   g_return_val_if_fail (nal_size != NULL, GST_H265_BIT_WRITER_ERROR);
   g_return_val_if_fail (*nal_size > 0, GST_H265_BIT_WRITER_ERROR);
@@ -2285,7 +2288,7 @@ gst_h265_bit_writer_convert_to_nal (guint nal_prefix_size,
 
   nal_writer_init (&nw, nal_prefix_size, packetized);
 
-  if (!nal_writer_put_bytes (&nw, raw_data, (guint) (raw_size / 8)))
+  if (!nal_writer_put_bytes (&nw, raw_data, raw_size / 8))
     goto error;
 
   if (raw_size % 8) {
