@@ -28,6 +28,9 @@
  * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
  * Boston, MA 02110-1301, USA.
  */
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include <string.h>
 #include <stdlib.h>
@@ -35,37 +38,58 @@
 #include "mpegts.h"
 #include "gstmpegts-private.h"
 
-/**
- * SECTION:gstmpegts
- * @title: Mpeg-ts helper library
- * @short_description: Mpeg-ts helper library for plugins and applications
- * @include: gst/mpegts/mpegts.h
- */
-
 /**
  * SECTION:gstmpegtssection
  * @title: Base MPEG-TS sections
- * @short_description: Sections for ITU H.222.0 | ISO/IEC 13818-1 
+ * @short_description: Sections for ITU H.222.0 | ISO/IEC 13818-1
  * @include: gst/mpegts/mpegts.h
  *
+ * ## Generic usage of sections with %GstMpegtsSection
+ *
+ * The %GstMpegtsSection object is the representation of MPEG-TS Section (SI or
+ * PSI).
+ *
+ * Various elements can post those on the bus via %GstMessage of type
+ * %GST_MESSAGE_ELEMENT. The gst_message_parse_mpegts_section() function
+ * provides access to the section.
+ *
+ * Applications (or other elements) can create them either by using one of the
+ * `gst_mpegts_section_from_*` functions, or by providing the raw SI data via
+ * gst_mpegts_section_new().
+ *
+ * Elements outputting MPEG-TS streams can also create sections using the
+ * various convenience functions and then get the packetized data (to be
+ * inserted in MPEG-TS packets) using gst_mpegts_section_packetize().
+ *
  * For more details, refer to the ITU H.222.0 or ISO/IEC 13818-1 specifications
  * and other specifications mentioned in the documentation.
- */
-
-/*
- * TODO
  *
- * * Check minimum size for section parsing in the various 
- *   gst_mpegts_section_get_<tabld>() methods
+ * # Supported base MPEG-TS sections
+ * These are the sections for which parsing and packetizing code exists.
  *
- * * Implement parsing code for
- *   * BAT
- *   * CAT
- *   * TSDT
+ * ## Program Association Table (PAT)
+ * See:
+ * * gst_mpegts_section_get_pat()
+ * * gst_mpegts_pat_program_new()
+ * * %GstMpegtsPatProgram
+ *
+ * ## Conditional Access Table (CAT)
+ * See:
+ * * gst_mpegts_section_get_cat()
+ *
+ * ## Program Map Table (PMT)
+ * See:
+ * * %GstMpegtsPMT
+ * * gst_mpegts_section_get_pmt()
+ * * gst_mpegts_pmt_new()
+ * * %GstMpegtsPMTStream
+ *
+ * ## Transport Stream Description Table (TSDT)
+ * See:
+ * * gst_mpegts_section_get_tsdt()
+ * # API
  */
 
-GST_DEBUG_CATEGORY (gst_mpegts_debug);
-
 static GQuark QUARK_PAT;
 static GQuark QUARK_CAT;
 static GQuark QUARK_BAT;
@@ -75,6 +99,7 @@ static GQuark QUARK_SDT;
 static GQuark QUARK_EIT;
 static GQuark QUARK_TDT;
 static GQuark QUARK_TOT;
+static GQuark QUARK_SCTE_SIT;
 static GQuark QUARK_SECTION;
 
 static GType _gst_mpegts_section_type = 0;
@@ -186,7 +211,7 @@ _gst_mpegts_section_free (GstMpegtsSection * section)
 
   g_free (section->data);
 
-  g_slice_free (GstMpegtsSection, section);
+  g_free (section);
 }
 
 static GstMpegtsSection *
@@ -194,7 +219,7 @@ _gst_mpegts_section_copy (GstMpegtsSection * section)
 {
   GstMpegtsSection *copy;
 
-  copy = g_slice_new0 (GstMpegtsSection);
+  copy = g_new0 (GstMpegtsSection, 1);
   gst_mini_object_init (GST_MINI_OBJECT_CAST (copy), 0, MPEG_TYPE_TS_SECTION,
       (GstMiniObjectCopyFunction) _gst_mpegts_section_copy, NULL,
       (GstMiniObjectFreeFunction) _gst_mpegts_section_free);
@@ -209,7 +234,7 @@ _gst_mpegts_section_copy (GstMpegtsSection * section)
   copy->last_section_number = section->last_section_number;
   copy->crc = section->crc;
 
-  copy->data = g_memdup (section->data, section->section_length);
+  copy->data = g_memdup2 (section->data, section->section_length);
   copy->section_length = section->section_length;
   /* Note: We do not copy the cached parsed item, it will be
    * reconstructed on that copy */
@@ -294,6 +319,9 @@ _mpegts_section_get_structure (GstMpegtsSection * section)
     case GST_MPEGTS_SECTION_TOT:
       quark = QUARK_TOT;
       break;
+    case GST_MPEGTS_SECTION_SCTE_SIT:
+      quark = QUARK_SCTE_SIT;
+      break;
     default:
       GST_DEBUG ("Creating structure for unknown GstMpegtsSection");
       quark = QUARK_SECTION;
@@ -329,8 +357,17 @@ gst_message_new_mpegts_section (GstObject * parent, GstMpegtsSection * section)
   return msg;
 }
 
-static GstEvent *
-_mpegts_section_get_event (GstMpegtsSection * section)
+/**
+ * gst_event_new_mpegts_section:
+ * @section: (transfer none): The #GstMpegtsSection to put in a message
+ *
+ * Creates a new #GstEvent for a #GstMpegtsSection.
+ *
+ * Returns: (transfer full): The new custom #GstEvent.
+ * Since: 1.20
+ */
+GstEvent *
+gst_event_new_mpegts_section (GstMpegtsSection * section)
 {
   GstStructure *structure;
   GstEvent *event;
@@ -348,7 +385,8 @@ _mpegts_section_get_event (GstMpegtsSection * section)
  *
  * Extracts the #GstMpegtsSection contained in the @event #GstEvent
  *
- * Returns: (transfer full): The extracted #GstMpegtsSection
+ * Returns: (transfer full): The extracted #GstMpegtsSection , or %NULL if the
+ * event did not contain a valid #GstMpegtsSection.
  */
 GstMpegtsSection *
 gst_event_parse_mpegts_section (GstEvent * event)
@@ -358,6 +396,9 @@ gst_event_parse_mpegts_section (GstEvent * event)
 
   structure = gst_event_get_structure (event);
 
+  if (!structure)
+    return NULL;
+
   if (!gst_structure_id_get (structure, QUARK_SECTION, MPEG_TYPE_TS_SECTION,
           &section, NULL))
     return NULL;
@@ -370,10 +411,10 @@ gst_event_parse_mpegts_section (GstEvent * event)
  * @element: (transfer none): The #GstElement to send to section event to
  * @section: (transfer none): The #GstMpegtsSection to put in the event
  *
- * Creates a custom #GstEvent with a @GstMpegtsSection.
- * The #GstEvent is sent to the @element #GstElement.
+ * Creates a custom #GstEvent with a @GstMpegtsSection and send it the @element
+ * #GstElement.
  *
- * Returns: %TRUE if the event is sent
+ * Returns: %TRUE if the event was sent to the element.
  */
 gboolean
 gst_mpegts_section_send_event (GstMpegtsSection * section, GstElement * element)
@@ -383,7 +424,7 @@ gst_mpegts_section_send_event (GstMpegtsSection * section, GstElement * element)
   g_return_val_if_fail (section != NULL, FALSE);
   g_return_val_if_fail (element != NULL, FALSE);
 
-  event = _mpegts_section_get_event (section);
+  event = gst_event_new_mpegts_section (section);
 
   if (!gst_element_send_event (element, event)) {
     gst_event_unref (event);
@@ -396,13 +437,13 @@ gst_mpegts_section_send_event (GstMpegtsSection * section, GstElement * element)
 static GstMpegtsPatProgram *
 _mpegts_pat_program_copy (GstMpegtsPatProgram * orig)
 {
-  return g_slice_dup (GstMpegtsPatProgram, orig);
+  return g_memdup2 (orig, sizeof (GstMpegtsPatProgram));
 }
 
 static void
 _mpegts_pat_program_free (GstMpegtsPatProgram * orig)
 {
-  g_slice_free (GstMpegtsPatProgram, orig);
+  g_free (orig);
 }
 
 G_DEFINE_BOXED_TYPE (GstMpegtsPatProgram, gst_mpegts_pat_program,
@@ -414,7 +455,7 @@ static gpointer
 _parse_pat (GstMpegtsSection * section)
 {
   GPtrArray *pat;
-  guint16 i = 0, nb_programs;
+  guint16 i, nb_programs;
   GstMpegtsPatProgram *program;
   guint8 *data, *end;
 
@@ -430,8 +471,10 @@ _parse_pat (GstMpegtsSection * section)
       g_ptr_array_new_full (nb_programs,
       (GDestroyNotify) _mpegts_pat_program_free);
 
-  while (data < end - 4) {
-    program = g_slice_new0 (GstMpegtsPatProgram);
+  GST_LOG ("nb_programs %u", nb_programs);
+
+  for (i = 0; i < nb_programs; i++) {
+    program = g_new0 (GstMpegtsPatProgram, 1);
     program->program_number = GST_READ_UINT16_BE (data);
     data += 2;
 
@@ -439,8 +482,6 @@ _parse_pat (GstMpegtsSection * section)
     data += 2;
 
     g_ptr_array_index (pat, i) = program;
-
-    i++;
   }
   pat->len = nb_programs;
 
@@ -462,12 +503,13 @@ _parse_pat (GstMpegtsSection * section)
  *
  * Returns the array of #GstMpegtsPatProgram contained in the section.
  *
- * Note: The PAT "transport_id" field corresponds to the "subtable_extension"
- * field of the provided @section.
+ * Note: The PAT `transport_stream_id` field corresponds to the
+ * "subtable_extension" field of the provided @section.
  *
  * Returns: (transfer container) (element-type GstMpegtsPatProgram): The
- * #GstMpegtsPatProgram contained in the section, or %NULL if an error
- * happened. Release with #g_ptr_array_unref when done.
+ * #GstMpegtsPatProgram contained in the section, or %NULL if an error happened
+ * or the @section did not contain a valid PAT. Release with #g_ptr_array_unref
+ * when done.
  */
 GPtrArray *
 gst_mpegts_section_get_pat (GstMpegtsSection * section)
@@ -488,7 +530,8 @@ gst_mpegts_section_get_pat (GstMpegtsSection * section)
 /**
  * gst_mpegts_pat_new:
  *
- * Allocates a new #GPtrArray for #GstMpegtsPatProgram
+ * Allocates a new #GPtrArray for #GstMpegtsPatProgram. The array can be filled
+ * and then converted to a PAT section with gst_mpegts_section_from_pat().
  *
  * Returns: (transfer full) (element-type GstMpegtsPatProgram): A newly allocated #GPtrArray
  */
@@ -515,7 +558,7 @@ gst_mpegts_pat_program_new (void)
 {
   GstMpegtsPatProgram *program;
 
-  program = g_slice_new0 (GstMpegtsPatProgram);
+  program = g_new0 (GstMpegtsPatProgram, 1);
 
   return program;
 }
@@ -596,7 +639,7 @@ _gst_mpegts_pmt_stream_copy (GstMpegtsPMTStream * pmt)
 {
   GstMpegtsPMTStream *copy;
 
-  copy = g_slice_dup (GstMpegtsPMTStream, pmt);
+  copy = g_memdup2 (pmt, sizeof (GstMpegtsPMTStream));
   copy->descriptors = g_ptr_array_ref (pmt->descriptors);
 
   return copy;
@@ -607,7 +650,7 @@ _gst_mpegts_pmt_stream_free (GstMpegtsPMTStream * pmt)
 {
   if (pmt->descriptors)
     g_ptr_array_unref (pmt->descriptors);
-  g_slice_free (GstMpegtsPMTStream, pmt);
+  g_free (pmt);
 }
 
 G_DEFINE_BOXED_TYPE (GstMpegtsPMTStream, gst_mpegts_pmt_stream,
@@ -619,7 +662,7 @@ _gst_mpegts_pmt_copy (GstMpegtsPMT * pmt)
 {
   GstMpegtsPMT *copy;
 
-  copy = g_slice_dup (GstMpegtsPMT, pmt);
+  copy = g_memdup2 (pmt, sizeof (GstMpegtsPMT));
   if (pmt->descriptors)
     copy->descriptors = g_ptr_array_ref (pmt->descriptors);
   copy->streams = g_ptr_array_ref (pmt->streams);
@@ -632,8 +675,9 @@ _gst_mpegts_pmt_free (GstMpegtsPMT * pmt)
 {
   if (pmt->descriptors)
     g_ptr_array_unref (pmt->descriptors);
-  g_ptr_array_unref (pmt->streams);
-  g_slice_free (GstMpegtsPMT, pmt);
+  if (pmt->streams)
+    g_ptr_array_unref (pmt->streams);
+  g_free (pmt);
 }
 
 G_DEFINE_BOXED_TYPE (GstMpegtsPMT, gst_mpegts_pmt,
@@ -649,7 +693,7 @@ _parse_pmt (GstMpegtsSection * section)
   guint program_info_length;
   guint stream_info_length;
 
-  pmt = g_slice_new0 (GstMpegtsPMT);
+  pmt = g_new0 (GstMpegtsPMT, 1);
 
   data = section->data;
   end = data + section->section_length;
@@ -686,7 +730,7 @@ _parse_pmt (GstMpegtsSection * section)
   /* parse entries, cycle until there's space for another entry (at least 5
    * bytes) plus the CRC */
   while (data <= end - 4 - 5) {
-    GstMpegtsPMTStream *stream = g_slice_new0 (GstMpegtsPMTStream);
+    GstMpegtsPMTStream *stream = g_new0 (GstMpegtsPMTStream, 1);
 
     g_ptr_array_add (pmt->streams, stream);
 
@@ -714,6 +758,11 @@ _parse_pmt (GstMpegtsSection * section)
     i += 1;
   }
 
+  /* Section length was longer than the actual content of the PMT */
+  if (data < end - 4)
+    goto error;
+
+  /* Ensure we did not read after the end of our array */
   g_assert (data == end - 4);
 
   return (gpointer) pmt;
@@ -729,7 +778,7 @@ error:
  * gst_mpegts_section_get_pmt:
  * @section: a #GstMpegtsSection of type %GST_MPEGTS_SECTION_PMT
  *
- * Returns the #GstMpegtsPMT contained in the @section.
+ * Parses the Program Map Table contained in the @section.
  *
  * Returns: The #GstMpegtsPMT contained in the section, or %NULL if an error
  * happened.
@@ -751,7 +800,9 @@ gst_mpegts_section_get_pmt (GstMpegtsSection * section)
 /**
  * gst_mpegts_pmt_new:
  *
- * Allocates and initializes a new #GstMpegtsPMT.
+ * Allocates and initializes a new #GstMpegtsPMT. #GstMpegtsPMTStream can be
+ * added to the streams array, and global PMT #GstMpegtsDescriptor to the
+ * descriptors array.
  *
  * Returns: (transfer full): #GstMpegtsPMT
  */
@@ -760,7 +811,7 @@ gst_mpegts_pmt_new (void)
 {
   GstMpegtsPMT *pmt;
 
-  pmt = g_slice_new0 (GstMpegtsPMT);
+  pmt = g_new0 (GstMpegtsPMT, 1);
 
   pmt->descriptors = g_ptr_array_new_with_free_func ((GDestroyNotify)
       gst_mpegts_descriptor_free);
@@ -782,7 +833,7 @@ gst_mpegts_pmt_stream_new (void)
 {
   GstMpegtsPMTStream *stream;
 
-  stream = g_slice_new0 (GstMpegtsPMTStream);
+  stream = g_new0 (GstMpegtsPMTStream, 1);
 
   stream->descriptors = g_ptr_array_new_with_free_func ((GDestroyNotify)
       gst_mpegts_descriptor_free);
@@ -929,11 +980,13 @@ _parse_cat (GstMpegtsSection * section)
  * gst_mpegts_section_get_cat:
  * @section: a #GstMpegtsSection of type %GST_MPEGTS_SECTION_CAT
  *
+ * Parses a Conditional Access Table.
+ *
  * Returns the array of #GstMpegtsDescriptor contained in the Conditional
  * Access Table.
  *
- * Returns: (transfer container) (element-type GstMpegtsDescriptor): The
- * #GstMpegtsDescriptor contained in the section, or %NULL if an error
+ * Returns: (transfer container) (element-type GstMpegtsDescriptor): The array
+ * of #GstMpegtsDescriptor contained in the section, or %NULL if an error
  * happened. Release with #g_array_unref when done.
  */
 GPtrArray *
@@ -957,10 +1010,12 @@ gst_mpegts_section_get_cat (GstMpegtsSection * section)
  * gst_mpegts_section_get_tsdt:
  * @section: a #GstMpegtsSection of type %GST_MPEGTS_SECTION_TSDT
  *
+ * Parses a Transport Stream Description Table.
+ *
  * Returns the array of #GstMpegtsDescriptor contained in the section
  *
- * Returns: (transfer container) (element-type GstMpegtsDescriptor): The
- * #GstMpegtsDescriptor contained in the section, or %NULL if an error
+ * Returns: (transfer container) (element-type GstMpegtsDescriptor): The array
+ * of #GstMpegtsDescriptor contained in the section, or %NULL if an error
  * happened. Release with #g_array_unref when done.
  */
 GPtrArray *
@@ -977,40 +1032,7 @@ gst_mpegts_section_get_tsdt (GstMpegtsSection * section)
 }
 
 
-/**
- * gst_mpegts_initialize:
- *
- * Initializes the MPEG-TS helper library. Must be called before any
- * usage.
- */
-void
-gst_mpegts_initialize (void)
-{
-  if (_gst_mpegts_section_type)
-    return;
 
-  GST_DEBUG_CATEGORY_INIT (gst_mpegts_debug, "mpegts", 0,
-      "MPEG-TS helper library");
-
-  /* FIXME : Temporary hack to initialize section gtype */
-  _gst_mpegts_section_type = gst_mpegts_section_get_type ();
-
-  QUARK_PAT = g_quark_from_string ("pat");
-  QUARK_CAT = g_quark_from_string ("cat");
-  QUARK_PMT = g_quark_from_string ("pmt");
-  QUARK_NIT = g_quark_from_string ("nit");
-  QUARK_BAT = g_quark_from_string ("bat");
-  QUARK_SDT = g_quark_from_string ("sdt");
-  QUARK_EIT = g_quark_from_string ("eit");
-  QUARK_TDT = g_quark_from_string ("tdt");
-  QUARK_TOT = g_quark_from_string ("tot");
-  QUARK_SECTION = g_quark_from_string ("section");
-
-  __initialize_descriptors ();
-}
-
-/* FIXME : Later on we might need to use more than just the table_id
- * to figure out which type of section this is. */
 static GstMpegtsSectionType
 _identify_section (guint16 pid, guint8 table_id)
 {
@@ -1070,6 +1092,16 @@ _identify_section (guint16 pid, guint8 table_id)
       if (pid == 0x1ffb)
         return GST_MPEGTS_SECTION_ATSC_STT;
       break;
+    case GST_MTS_TABLE_ID_ATSC_RATING_REGION:
+      if (pid == 0x1ffb)
+        return GST_MPEGTS_SECTION_ATSC_RRT;
+      break;
+    case GST_MTS_TABLE_ID_SCTE_SPLICE:
+      return GST_MPEGTS_SECTION_SCTE_SIT;
+      break;
+    case GST_MTS_TABLE_ID_SELECTION_INFORMATION:
+      if (pid == 0x001f)
+        return GST_MPEGTS_SECTION_SIT;
     default:
       /* Handle ranges */
       if (table_id >= GST_MTS_TABLE_ID_EVENT_INFORMATION_ACTUAL_TS_PRESENT &&
@@ -1088,7 +1120,7 @@ _gst_mpegts_section_init (guint16 pid, guint8 table_id)
 {
   GstMpegtsSection *section;
 
-  section = g_slice_new0 (GstMpegtsSection);
+  section = g_new0 (GstMpegtsSection, 1);
   gst_mini_object_init (GST_MINI_OBJECT_CAST (section), 0, MPEG_TYPE_TS_SECTION,
       (GstMiniObjectCopyFunction) _gst_mpegts_section_copy, NULL,
       (GstMiniObjectFreeFunction) _gst_mpegts_section_free);
@@ -1120,8 +1152,13 @@ _packetize_common_section (GstMpegtsSection * section, gsize length)
     case GST_MPEGTS_SECTION_PMT:
     case GST_MPEGTS_SECTION_CAT:
     case GST_MPEGTS_SECTION_TSDT:
+    case GST_MPEGTS_SECTION_SCTE_SIT:
       /* Tables from ISO/IEC 13818-1 has a '0' bit
        * after the section_syntax_indicator */
+      /* FIXME : that 'bit' after the section_syntax_indicator is the
+       * private_indicator field. We should figure out why/when it
+       * needs to be set *OR* decide that by default it is set to 0
+       * (i.e. not private data) unless the user/caller decides */
       GST_WRITE_UINT16_BE (data, (section->section_length - 3) | 0x3000);
       break;
     default:
@@ -1156,8 +1193,8 @@ _packetize_common_section (GstMpegtsSection * section, gsize length)
 /**
  * gst_mpegts_section_new:
  * @pid: the PID to which this section belongs
- * @data: (transfer full): a pointer to the beginning of the section (i.e. the first byte
- * should contain the table_id field).
+ * @data: (transfer full) (array length=data_size): a pointer to the beginning of
+ * the section (i.e. the first byte should contain the `table_id` field).
  * @data_size: size of the @data argument.
  *
  * Creates a new #GstMpegtsSection from the provided @data.
@@ -1178,13 +1215,20 @@ gst_mpegts_section_new (guint16 pid, guint8 * data, gsize data_size)
   GstMpegtsSection *res = NULL;
   guint8 tmp;
   guint8 table_id;
-  guint16 section_length;
+  guint16 section_length = 0;
+
+  /* The smallest section ever is 3 bytes */
+  if (G_UNLIKELY (data_size < 3))
+    goto short_packet;
 
   /* Check for length */
   section_length = GST_READ_UINT16_BE (data + 1) & 0x0FFF;
   if (G_UNLIKELY (data_size < section_length + 3))
     goto short_packet;
 
+  GST_LOG ("data_size:%" G_GSIZE_FORMAT " section_length:%u",
+      data_size, section_length);
+
   /* Table id is in first byte */
   table_id = *data;
 
@@ -1199,6 +1243,13 @@ gst_mpegts_section_new (guint16 pid, guint8 * data, gsize data_size)
   /* section_length (already parsed) : 12 bit */
   res->section_length = section_length + 3;
   if (!res->short_section) {
+    /* A long packet needs to be at least 11 bytes long
+     * _ 3 for the bytes above
+     * _ 5 for the bytes below
+     * _ 4 for the CRC */
+    if (G_UNLIKELY (data_size < 11))
+      goto bad_long_packet;
+
     /* CRC is after section_length (-4 for the size of the CRC) */
     res->crc = GST_READ_UINT32_BE (res->data + res->section_length - 4);
     /* Skip to after section_length */
@@ -1228,6 +1279,13 @@ short_packet:
     g_free (data);
     return NULL;
   }
+bad_long_packet:
+  {
+    GST_WARNING ("PID 0x%04x long section is too short (%" G_GSIZE_FORMAT
+        " bytes, need at least 11)", pid, data_size);
+    gst_mpegts_section_unref (res);
+    return NULL;
+  }
 }
 
 /**
@@ -1235,10 +1293,11 @@ short_packet:
  * @section: (transfer none): the #GstMpegtsSection that holds the data
  * @output_size: (out): #gsize to hold the size of the data
  *
- * If the data in @section has already been packetized, the data pointer is returned
- * immediately. Otherwise, the data field is allocated and populated.
+ * Packetize (i.e. serialize) the @section. If the data in @section has already
+ * been packetized, the data pointer is returned immediately. Otherwise, the
+ * data field is allocated and populated.
  *
- * Returns: (transfer none): pointer to section data, or %NULL on fail
+ * Returns: (transfer none): pointer to section data, or %NULL on failure.
  */
 guint8 *
 gst_mpegts_section_packetize (GstMpegtsSection * section, gsize * output_size)
@@ -1246,7 +1305,6 @@ gst_mpegts_section_packetize (GstMpegtsSection * section, gsize * output_size)
   guint8 *crc;
   g_return_val_if_fail (section != NULL, NULL);
   g_return_val_if_fail (output_size != NULL, NULL);
-  g_return_val_if_fail (section->packetizer != NULL, NULL);
 
   /* Section data has already been packetized */
   if (section->data) {
@@ -1254,6 +1312,8 @@ gst_mpegts_section_packetize (GstMpegtsSection * section, gsize * output_size)
     return section->data;
   }
 
+  g_return_val_if_fail (section->packetizer != NULL, NULL);
+
   if (!section->packetizer (section))
     return NULL;
 
@@ -1267,3 +1327,22 @@ gst_mpegts_section_packetize (GstMpegtsSection * section, gsize * output_size)
 
   return section->data;
 }
+
+void
+__initialize_sections (void)
+{
+  /* FIXME : Temporary hack to initialize section gtype */
+  _gst_mpegts_section_type = gst_mpegts_section_get_type ();
+
+  QUARK_PAT = g_quark_from_string ("pat");
+  QUARK_CAT = g_quark_from_string ("cat");
+  QUARK_PMT = g_quark_from_string ("pmt");
+  QUARK_NIT = g_quark_from_string ("nit");
+  QUARK_BAT = g_quark_from_string ("bat");
+  QUARK_SDT = g_quark_from_string ("sdt");
+  QUARK_EIT = g_quark_from_string ("eit");
+  QUARK_TDT = g_quark_from_string ("tdt");
+  QUARK_TOT = g_quark_from_string ("tot");
+  QUARK_SCTE_SIT = g_quark_from_string ("scte-sit");
+  QUARK_SECTION = g_quark_from_string ("section");
+}
