@@ -23,7 +23,6 @@
 #endif
 
 #include "gstvkformat.h"
-#include "gstvkphysicaldevice-private.h"
 
 /**
  * SECTION:vkformat
@@ -173,7 +172,9 @@ static GstVulkanFormatInfo formats[] = {
   {FORMAT (R8, UINT), FLAG (RGB) | NE, DPTH8, PSTR4, PLANE0, OFFS0, SUB4, VK_IMAGE_ASPECT_COLOR_BIT},
   {FORMAT (R8, SINT), FLAG (RGB) | NE, DPTH8, PSTR4, PLANE0, OFFS0, SUB4, VK_IMAGE_ASPECT_COLOR_BIT},
   {FORMAT (R8, SRGB), FLAG (RGB) | NE, DPTH8, PSTR4, PLANE0, OFFS0, SUB4, VK_IMAGE_ASPECT_COLOR_BIT},
+#if (defined(VK_VERSION_1_3) || defined(VK_VERSION_1_2) && VK_HEADER_VERSION >= 199)
   {FORMAT (G8_B8R8_2PLANE_420, UNORM), FLAG (YUV), DPTH888, PSTR122, PLANE011, OFFS001, SUB420, ASPECT_2PLANE},
+#endif
 #if 0
 FIXME: implement:
   {VK_FORMAT_R4G4_UNORM_PACK8, {0, 1, -1, -1}},
@@ -431,121 +432,49 @@ gst_vulkan_format_get_aspect (VkFormat format)
   return 0;                     /* VK_IMAGE_ASPECT_NONE */
 }
 
-/* The in-memory ordering of bytes within a component is determined by the host
- * endianness. */
-#if G_BYTE_ORDER == G_LITTLE_ENDIAN
-#define GST_VIDEO_FORMAT_ENDIANNESS(fmt) G_PASTE(G_PASTE(GST_VIDEO_FORMAT_, fmt), LE)
-#else
-#define GST_VIDEO_FORMAT_ENDIANNESS(fmt) G_PASTE(G_PASTE(GST_VIDEO_FORMAT_, fmt), BE)
-#endif
-
 /* *INDENT-OFF* */
-const static GstVulkanFormatMap vk_formats_map[] = {
-  /* RGB                   unsigned normalized format         sRGB nonlinear encoding */
-  { GST_VIDEO_FORMAT_RGBA,  VK_FORMAT_R8G8B8A8_UNORM,      { VK_FORMAT_R8G8B8A8_SRGB, } },
-  { GST_VIDEO_FORMAT_RGBx,  VK_FORMAT_R8G8B8A8_UNORM,      { VK_FORMAT_R8G8B8A8_SRGB, } },
-  { GST_VIDEO_FORMAT_BGRA,  VK_FORMAT_B8G8R8A8_UNORM,      { VK_FORMAT_B8G8R8A8_SRGB, } },
-  { GST_VIDEO_FORMAT_BGRx,  VK_FORMAT_B8G8R8A8_UNORM,      { VK_FORMAT_B8G8R8A8_SRGB, } },
-  { GST_VIDEO_FORMAT_ARGB,  VK_FORMAT_R8G8B8A8_UNORM,      { VK_FORMAT_UNDEFINED, } },
-  { GST_VIDEO_FORMAT_xRGB,  VK_FORMAT_R8G8B8A8_UNORM,      { VK_FORMAT_UNDEFINED, } },
-  { GST_VIDEO_FORMAT_ABGR,  VK_FORMAT_R8G8B8A8_UNORM,      { VK_FORMAT_UNDEFINED, } },
-  { GST_VIDEO_FORMAT_xBGR,  VK_FORMAT_R8G8B8A8_UNORM,      { VK_FORMAT_UNDEFINED, } },
+const static struct {
+  GstVideoFormat format;
+  VkFormat vkfrmt;
+  VkFormat vkfrmts[GST_VIDEO_MAX_PLANES];
+} vk_formats_map[] = {
+  /* RGB                                                     transfer sRGB */
+  { GST_VIDEO_FORMAT_RGBA,  VK_FORMAT_R8G8B8A8_SRGB,       { VK_FORMAT_R8G8B8A8_UNORM, } },
+  { GST_VIDEO_FORMAT_RGBx,  VK_FORMAT_R8G8B8A8_SRGB,       { VK_FORMAT_R8G8B8A8_UNORM, } },
+  { GST_VIDEO_FORMAT_BGRA,  VK_FORMAT_B8G8R8A8_SRGB,       { VK_FORMAT_B8G8R8A8_UNORM, } },
+  { GST_VIDEO_FORMAT_BGRx,  VK_FORMAT_B8G8R8A8_SRGB,       { VK_FORMAT_B8G8R8A8_UNORM, } },
+  { GST_VIDEO_FORMAT_ARGB,  VK_FORMAT_UNDEFINED,           { VK_FORMAT_R8G8B8A8_UNORM, } },
+  { GST_VIDEO_FORMAT_xRGB,  VK_FORMAT_UNDEFINED,           { VK_FORMAT_R8G8B8A8_UNORM, } },
+  { GST_VIDEO_FORMAT_ABGR,  VK_FORMAT_UNDEFINED,           { VK_FORMAT_R8G8B8A8_UNORM, } },
+  { GST_VIDEO_FORMAT_xBGR,  VK_FORMAT_UNDEFINED,           { VK_FORMAT_R8G8B8A8_UNORM, } },
   { GST_VIDEO_FORMAT_RGB,   VK_FORMAT_R8G8B8_UNORM,        { VK_FORMAT_UNDEFINED, } },
   { GST_VIDEO_FORMAT_BGR,   VK_FORMAT_B8G8R8_UNORM,        { VK_FORMAT_UNDEFINED, } },
   { GST_VIDEO_FORMAT_RGB16, VK_FORMAT_R5G6B5_UNORM_PACK16, { VK_FORMAT_UNDEFINED, } },
   { GST_VIDEO_FORMAT_BGR16, VK_FORMAT_B5G6R5_UNORM_PACK16, { VK_FORMAT_UNDEFINED, } },
   /* Gray */
-  { GST_VIDEO_FORMAT_GRAY8,                VK_FORMAT_R8_UNORM,   { VK_FORMAT_R8_UNORM, } },
-  { GST_VIDEO_FORMAT_ENDIANNESS (GRAY16_), VK_FORMAT_R16_UNORM,  { VK_FORMAT_R16_UNORM, } },
-  /* From Vulkan 1.4.319 spec chapter 51.1.1 "Compatible Formats of Planes of
-   * Multi-Planar Formats" Table 72 */
+  { GST_VIDEO_FORMAT_GRAY16_BE, VK_FORMAT_R8G8_UNORM, { VK_FORMAT_UNDEFINED, } },
+  { GST_VIDEO_FORMAT_GRAY16_LE, VK_FORMAT_R8G8_UNORM, { VK_FORMAT_UNDEFINED, } },
+  { GST_VIDEO_FORMAT_GRAY8,     VK_FORMAT_R8_UNORM,   { VK_FORMAT_UNDEFINED, } },
   /* YUV                                               planes */
-  { GST_VIDEO_FORMAT_AYUV,                 VK_FORMAT_UNDEFINED,  { VK_FORMAT_R8G8B8A8_UNORM, } },
-  /* 1-plane 410 */
-  { GST_VIDEO_FORMAT_Y41B,                 VK_FORMAT_UNDEFINED, { VK_FORMAT_R8_UNORM, } },
-  /* 1-plane 422 */
-  { GST_VIDEO_FORMAT_YUY2,                 VK_FORMAT_G8B8G8R8_422_UNORM,                        { VK_FORMAT_R8G8B8A8_UNORM } },
-  { GST_VIDEO_FORMAT_UYVY,                 VK_FORMAT_B8G8R8G8_422_UNORM,                        { VK_FORMAT_R8G8B8A8_UNORM } },
-  { GST_VIDEO_FORMAT_Y210,                 VK_FORMAT_G10X6B10X6G10X6R10X6_422_UNORM_4PACK16,    { VK_FORMAT_R16G16B16A16_UNORM } },
-  { GST_VIDEO_FORMAT_ENDIANNESS (Y212_),   VK_FORMAT_G12X4B12X4G12X4R12X4_422_UNORM_4PACK16,    { VK_FORMAT_R16G16B16A16_UNORM } },
-  /* { XXX, VK_FORMAT_G16B16G16R16_422_UNORM, { VK_FORMAT_R16G16B16A16_UNORM } }, */
-  /* 1-plane 444 */
-  /* { XXX, VK_FORMAT_B8G8R8A8_UNORM, { VK_FORMAT_B8G8R8A8_UNORM     } }, */
-  { GST_VIDEO_FORMAT_Y410,                 VK_FORMAT_A2R10G10B10_UNORM_PACK32,                  { VK_FORMAT_R16G16B16A16_UNORM } },
-  { GST_VIDEO_FORMAT_ENDIANNESS (Y412_),   VK_FORMAT_R12X4G12X4B12X4A12X4_UNORM_4PACK16,        { VK_FORMAT_R16G16B16A16_UNORM } },
-  { GST_VIDEO_FORMAT_ENDIANNESS (Y416_),   VK_FORMAT_R16G16B16A16_UNORM,                        { VK_FORMAT_R16G16B16A16_UNORM } },
-  /* 2-planes 420 */
-  { GST_VIDEO_FORMAT_NV21,                 VK_FORMAT_UNDEFINED,                                 { VK_FORMAT_R8_UNORM, VK_FORMAT_R8G8_UNORM } },
-  { GST_VIDEO_FORMAT_NV12,                 VK_FORMAT_G8_B8R8_2PLANE_420_UNORM,                  { VK_FORMAT_R8_UNORM, VK_FORMAT_R8G8_UNORM } },
-  { GST_VIDEO_FORMAT_ENDIANNESS (P010_10), VK_FORMAT_G10X6_B10X6R10X6_2PLANE_420_UNORM_3PACK16, { VK_FORMAT_R10X6_UNORM_PACK16, VK_FORMAT_R10X6G10X6_UNORM_2PACK16 } },
-  { GST_VIDEO_FORMAT_ENDIANNESS (P012_),   VK_FORMAT_G12X4_B12X4R12X4_2PLANE_420_UNORM_3PACK16, { VK_FORMAT_R12X4_UNORM_PACK16, VK_FORMAT_R12X4G12X4_UNORM_2PACK16 } },
-  { GST_VIDEO_FORMAT_ENDIANNESS (P016_),   VK_FORMAT_G16_B16R16_2PLANE_420_UNORM,               { VK_FORMAT_R16_UNORM, VK_FORMAT_R16G16_UNORM } },
-  /* 2-planes 422 */
-  { GST_VIDEO_FORMAT_NV16,                 VK_FORMAT_G8_B8R8_2PLANE_422_UNORM,                  { VK_FORMAT_R8_UNORM,  VK_FORMAT_R8G8_UNORM } },
-  /* { GST_VIDEO_FORMAT_ENDIANNESS (P021_10), VK_FORMAT_G10X6_B10X6R10X6_2PLANE_422_UNORM_3PACK16, { VK_FORMAT_R16_UNORM, VK_FORMAT_R16G16_UNORM } }, */
-  /* { GST_VIDEO_FORMAT_ENDIANNESS (P021_12), VK_FORMAT_G12X4_B12X4R12X4_2PLANE_422_UNORM_3PACK16, { VK_FORMAT_R16_UNORM, VK_FORMAT_R16G16_UNORM } }, */
-  /* { GST_VIDEO_FORMAT_ENDIANNESS (P021_16), VK_FORMAT_G16_B16R16_2PLANE_422_UNORM,               { VK_FORMAT_R16_UNORM, VK_FORMAT_R16G16_UNORM } }, */
-  /* 2-planes 444 */
-  { GST_VIDEO_FORMAT_NV24,                 VK_FORMAT_G8_B8R8_2PLANE_444_UNORM,                  { VK_FORMAT_R8_UNORM, VK_FORMAT_R8G8_UNORM } },
-  /* { XXX, VK_FORMAT_G16_B16R16_2PLANE_444_UNORM, { VK_FORMAT_R16_UNORM, VK_FORMAT_R16G16_UNORM } } */
-  /* { XXX, VK_FORMAT_G10X6_B10X6R10X6_2PLANE_444_UNORM_3PACK16, { VK_FORMAT_R16_UNORM, VK_FORMAT_R16G16_UNORM } } */
-  /* { XXX, VK_FORMAT_G12X4_B12X4R12X4_2PLANE_444_UNORM_3PACK16, { VK_FORMAT_R16_UNORM, VK_FORMAT_R16G16_UNORM } } */
-  /* 3-planes 420 */
-  /* { GST_VIDEO_FORMAT_YV12, VK_FORMAT_UNDEFINED, { VK_FORMAT_R8_UNORM, } }, UV inverted I420 */
-  { GST_VIDEO_FORMAT_I420,                 VK_FORMAT_G8_B8_R8_3PLANE_420_UNORM,                 { VK_FORMAT_R8_UNORM,  VK_FORMAT_R8_UNORM,  VK_FORMAT_R8_UNORM  } },
-  { GST_VIDEO_FORMAT_ENDIANNESS (I420_10), VK_FORMAT_G16_B16_R16_3PLANE_420_UNORM,              { VK_FORMAT_R16_UNORM, VK_FORMAT_R16_UNORM, VK_FORMAT_R16_UNORM } },
-  { GST_VIDEO_FORMAT_ENDIANNESS (I420_12), VK_FORMAT_G16_B16_R16_3PLANE_420_UNORM,              { VK_FORMAT_R16_UNORM, VK_FORMAT_R16_UNORM, VK_FORMAT_R16_UNORM } },
-  /* { GST_VIDEO_FORMAT_ENDIANNESS (I420_16), VK_FORMAT_G16_B16_R16_3PLANE_420_UNORM, { VK_FORMAT_R16_UNORM, VK_FORMAT_R16_UNORM, VK_FORMAT_R16_UNORM } }, */
-  /* 3-planes 422 */
-  { GST_VIDEO_FORMAT_Y42B,                 VK_FORMAT_G8_B8_R8_3PLANE_422_UNORM,                 { VK_FORMAT_R8_UNORM,  VK_FORMAT_R8_UNORM,  VK_FORMAT_R8_UNORM  } },
-  { GST_VIDEO_FORMAT_ENDIANNESS (I422_10), VK_FORMAT_G16_B16_R16_3PLANE_422_UNORM,              { VK_FORMAT_R16_UNORM, VK_FORMAT_R16_UNORM, VK_FORMAT_R16_UNORM } },
-  { GST_VIDEO_FORMAT_ENDIANNESS (I422_12), VK_FORMAT_G16_B16_R16_3PLANE_422_UNORM,              { VK_FORMAT_R16_UNORM, VK_FORMAT_R16_UNORM, VK_FORMAT_R16_UNORM } },
-  /* { GST_VIDEO_FORMAT_ENDIANNESS (I422_16), VK_FORMAT_G16_B16_R16_3PLANE_422_UNORM, { VK_FORMAT_R16_UNORM, VK_FORMAT_R16_UNORM, VK_FORMAT_R16_UNORM } }, */
-  /* 3-planes 444 */
-  { GST_VIDEO_FORMAT_Y444,                 VK_FORMAT_G8_B8_R8_3PLANE_444_UNORM,                 { VK_FORMAT_R8_UNORM,  VK_FORMAT_R8_UNORM,  VK_FORMAT_R8_UNORM  } },
-  { GST_VIDEO_FORMAT_ENDIANNESS (Y444_10), VK_FORMAT_G16_B16_R16_3PLANE_444_UNORM,              { VK_FORMAT_R16_UNORM, VK_FORMAT_R16_UNORM, VK_FORMAT_R16_UNORM } },
-  { GST_VIDEO_FORMAT_ENDIANNESS (Y444_12), VK_FORMAT_G16_B16_R16_3PLANE_444_UNORM,              { VK_FORMAT_R16_UNORM, VK_FORMAT_R16_UNORM, VK_FORMAT_R16_UNORM } },
-  /* { GST_VIDEO_FORMAT_ENDIANNESS (Y444_16), VK_FORMAT_G16_B16_R16_3PLANE_444_UNORM, { VK_FORMAT_R16_UNORM, VK_FORMAT_R16_UNORM, VK_FORMAT_R16_UNORM } }, */
-  /* YUVA 420 */
-  { GST_VIDEO_FORMAT_A420,                 VK_FORMAT_R8_UNORM,  { VK_FORMAT_R8_UNORM,   VK_FORMAT_R8_UNORM,   VK_FORMAT_R8_UNORM,   VK_FORMAT_R8_UNORM   } },
-  { GST_VIDEO_FORMAT_ENDIANNESS (A420_10), VK_FORMAT_R16_UNORM, { VK_FORMAT_R16_UNORM,  VK_FORMAT_R16_UNORM,  VK_FORMAT_R16_UNORM,  VK_FORMAT_R16_UNORM  } },
-  { GST_VIDEO_FORMAT_ENDIANNESS (A420_12), VK_FORMAT_R16_UNORM, { VK_FORMAT_R16_UNORM,  VK_FORMAT_R16_UNORM,  VK_FORMAT_R16_UNORM,  VK_FORMAT_R16_UNORM  } },
-  { GST_VIDEO_FORMAT_ENDIANNESS (A420_16), VK_FORMAT_R16_UNORM, { VK_FORMAT_R16_UNORM,  VK_FORMAT_R16_UNORM,  VK_FORMAT_R16_UNORM,  VK_FORMAT_R16_UNORM  } },
-  /* YUVA 422 */
-  { GST_VIDEO_FORMAT_A422,                 VK_FORMAT_R8_UNORM,  { VK_FORMAT_R8_UNORM,   VK_FORMAT_R8_UNORM,   VK_FORMAT_R8_UNORM,   VK_FORMAT_R8_UNORM   } },
-  { GST_VIDEO_FORMAT_ENDIANNESS (A422_10), VK_FORMAT_R16_UNORM, { VK_FORMAT_R16_UNORM,  VK_FORMAT_R16_UNORM,  VK_FORMAT_R16_UNORM,  VK_FORMAT_R16_UNORM  } },
-  { GST_VIDEO_FORMAT_ENDIANNESS (A422_12), VK_FORMAT_R16_UNORM, { VK_FORMAT_R16_UNORM,  VK_FORMAT_R16_UNORM,  VK_FORMAT_R16_UNORM,  VK_FORMAT_R16_UNORM  } },
-  { GST_VIDEO_FORMAT_ENDIANNESS (A422_16), VK_FORMAT_R16_UNORM, { VK_FORMAT_R16_UNORM,  VK_FORMAT_R16_UNORM,  VK_FORMAT_R16_UNORM,  VK_FORMAT_R16_UNORM  } },
-  /* YUVA 444 */
-  { GST_VIDEO_FORMAT_A444,                 VK_FORMAT_R8_UNORM,  { VK_FORMAT_R8_UNORM,   VK_FORMAT_R8_UNORM,   VK_FORMAT_R8_UNORM,   VK_FORMAT_R8_UNORM   } },
-  { GST_VIDEO_FORMAT_ENDIANNESS (A444_10), VK_FORMAT_R16_UNORM, { VK_FORMAT_R16_UNORM,  VK_FORMAT_R16_UNORM,  VK_FORMAT_R16_UNORM,  VK_FORMAT_R16_UNORM  } },
-  { GST_VIDEO_FORMAT_ENDIANNESS (A444_12), VK_FORMAT_R16_UNORM, { VK_FORMAT_R16_UNORM,  VK_FORMAT_R16_UNORM,  VK_FORMAT_R16_UNORM,  VK_FORMAT_R16_UNORM  } },
-  { GST_VIDEO_FORMAT_ENDIANNESS (A444_16), VK_FORMAT_R16_UNORM, { VK_FORMAT_R16_UNORM,  VK_FORMAT_R16_UNORM,  VK_FORMAT_R16_UNORM,  VK_FORMAT_R16_UNORM  } },
+  { GST_VIDEO_FORMAT_AYUV, VK_FORMAT_UNDEFINED, { VK_FORMAT_R8G8B8A8_UNORM, } },
+  { GST_VIDEO_FORMAT_YUY2, VK_FORMAT_UNDEFINED, { VK_FORMAT_R8G8_UNORM, } },
+  { GST_VIDEO_FORMAT_UYVY, VK_FORMAT_UNDEFINED, { VK_FORMAT_R8G8_UNORM, } },
+  { GST_VIDEO_FORMAT_NV12,
+#if (defined(VK_VERSION_1_3) || defined(VK_VERSION_1_2) && VK_HEADER_VERSION >= 199)
+    VK_FORMAT_G8_B8R8_2PLANE_420_UNORM,
+#else
+    VK_FORMAT_UNDEFINED,
+#endif
+    { VK_FORMAT_R8_UNORM, VK_FORMAT_R8G8_UNORM } },
+  { GST_VIDEO_FORMAT_NV21, VK_FORMAT_UNDEFINED, { VK_FORMAT_R8_UNORM, VK_FORMAT_R8G8_UNORM } },
+  { GST_VIDEO_FORMAT_Y444, VK_FORMAT_UNDEFINED, { VK_FORMAT_R8_UNORM,  } },
+  { GST_VIDEO_FORMAT_Y42B, VK_FORMAT_UNDEFINED, { VK_FORMAT_R8_UNORM, } },
+  { GST_VIDEO_FORMAT_Y41B, VK_FORMAT_UNDEFINED, { VK_FORMAT_R8_UNORM, } },
+  { GST_VIDEO_FORMAT_I420, VK_FORMAT_UNDEFINED, { VK_FORMAT_R8_UNORM, } },
+  { GST_VIDEO_FORMAT_YV12, VK_FORMAT_UNDEFINED, { VK_FORMAT_R8_UNORM, } },
 };
 /* *INDENT-ON* */
 
-/**
- * gst_vulkan_format_get_map: (skip)
- * @format: the #GstVideoFormat to get
- *
- * Returns: (nullable): the #GstVulkanFormatMap matching with @format
- *
- * Since: 1.26
- */
-const GstVulkanFormatMap *
-gst_vulkan_format_get_map (GstVideoFormat format)
-{
-  guint i;
-
-  for (i = 0; i < G_N_ELEMENTS (vk_formats_map); i++) {
-    if (vk_formats_map[i].format != format)
-      continue;
-    return &vk_formats_map[i];
-  }
-
-  return NULL;
-}
-
 /**
  * gst_vulkan_format_from_video_info: (skip)
  * @v_info: the #GstVideoInfo
@@ -565,7 +494,12 @@ gst_vulkan_format_from_video_info (GstVideoInfo * v_info, guint plane)
       continue;
 
     if (GST_VIDEO_INFO_IS_RGB (v_info)) {
-      return vk_formats_map[i].vkfrmt;
+      if (GST_VIDEO_INFO_COLORIMETRY (v_info).transfer ==
+          GST_VIDEO_TRANSFER_SRGB) {
+        return vk_formats_map[i].vkfrmt;
+      } else {
+        return vk_formats_map[i].vkfrmts[0];
+      }
     } else if (GST_VIDEO_INFO_IS_YUV (v_info) &&
         GST_VIDEO_INFO_N_PLANES (v_info) > plane) {
       return vk_formats_map[i].vkfrmts[plane];
@@ -600,17 +534,17 @@ _get_usage (guint64 feature)
     {VK_FORMAT_FEATURE_STORAGE_IMAGE_BIT, VK_IMAGE_USAGE_STORAGE_BIT},
     {VK_FORMAT_FEATURE_COLOR_ATTACHMENT_BIT,
           VK_IMAGE_USAGE_COLOR_ATTACHMENT_BIT | VK_IMAGE_USAGE_INPUT_ATTACHMENT_BIT},
-#if defined(VK_KHR_video_decode_queue)
+#if GST_VULKAN_HAVE_VIDEO_EXTENSIONS
     {VK_FORMAT_FEATURE_2_VIDEO_DECODE_OUTPUT_BIT_KHR,
           VK_IMAGE_USAGE_VIDEO_DECODE_DST_BIT_KHR},
     {VK_FORMAT_FEATURE_2_VIDEO_DECODE_DPB_BIT_KHR,
           VK_IMAGE_USAGE_VIDEO_DECODE_DPB_BIT_KHR},
-#endif
-#if defined(VK_KHR_video_encode_queue)
+#ifdef VK_ENABLE_BETA_EXTENSIONS
     {VK_FORMAT_FEATURE_2_VIDEO_ENCODE_DPB_BIT_KHR,
           VK_IMAGE_USAGE_VIDEO_ENCODE_DPB_BIT_KHR},
     {VK_FORMAT_FEATURE_2_VIDEO_ENCODE_INPUT_BIT_KHR,
           VK_IMAGE_USAGE_VIDEO_ENCODE_SRC_BIT_KHR},
+#endif
 #endif
   };
   /* *INDENT-ON* */
@@ -623,9 +557,49 @@ _get_usage (guint64 feature)
   return usage;
 }
 
+static guint64
+_get_feature_flags (VkPhysicalDevice gpu, gpointer func, VkFormat format,
+    VkImageTiling tiling)
+{
+  VkFormatProperties prop = { 0 };
+#if defined (VK_KHR_get_physical_device_properties2)
+#if defined (VK_KHR_format_feature_flags2)
+  VkFormatProperties3KHR prop3 = {
+    .sType = VK_STRUCTURE_TYPE_FORMAT_PROPERTIES_3_KHR,
+  };
+#endif
+  VkFormatProperties2KHR prop2 = {
+    .sType = VK_STRUCTURE_TYPE_FORMAT_PROPERTIES_2_KHR,
+#if defined (VK_KHR_format_feature_flags2)
+    .pNext = &prop3,
+#endif
+  };
+
+  if (func) {
+    PFN_vkGetPhysicalDeviceFormatProperties2KHR
+        gst_vkGetPhysicalDeviceFormatProperties2 = func;
+
+    gst_vkGetPhysicalDeviceFormatProperties2 (gpu, format, &prop2);
+#if defined (VK_KHR_format_feature_flags2)
+    return tiling == VK_IMAGE_TILING_LINEAR ?
+        prop3.linearTilingFeatures : prop3.optimalTilingFeatures;
+#else
+    return tiling == VK_IMAGE_TILING_LINEAR ?
+        prop2.formatProperties.linearTilingFeatures :
+        prop2.formatProperties.optimalTilingFeatures;
+#endif
+  }
+#endif /* defined (VK_KHR_get_physical_device_properties2) */
+
+  /* fallback */
+  vkGetPhysicalDeviceFormatProperties (gpu, format, &prop);
+  return tiling == VK_IMAGE_TILING_LINEAR ?
+      prop.linearTilingFeatures : prop.optimalTilingFeatures;
+}
+
 /**
  * gst_vulkan_format_from_video_info_2: (skip)
- * @device: a #GstVulkanDevice
+ * @physical_device: a #GstVulkanPhysicalDevice
  * @info: the #GstVideoInfo
  * @tiling: the tiling to use
  * @no_multiplane: query for vulkan formats without multiple images
@@ -639,13 +613,29 @@ _get_usage (guint64 feature)
  * Since: 1.24
  */
 gboolean
-gst_vulkan_format_from_video_info_2 (GstVulkanDevice * device,
+gst_vulkan_format_from_video_info_2 (GstVulkanPhysicalDevice * physical_device,
     GstVideoInfo * info, VkImageTiling tiling, gboolean no_multiplane,
     VkImageUsageFlags requested_usage, VkFormat fmts[GST_VIDEO_MAX_PLANES],
     int *n_imgs, VkImageUsageFlags * usage_ret)
 {
   int i;
-  GstVulkanFormatProperties props = { 0, };
+  VkPhysicalDevice gpu;
+#if defined (VK_KHR_get_physical_device_properties2)
+  PFN_vkGetPhysicalDeviceFormatProperties2KHR
+      gst_vkGetPhysicalDeviceFormatProperties2 = NULL;
+
+  gst_vkGetPhysicalDeviceFormatProperties2 =
+      gst_vulkan_instance_get_proc_address (physical_device->instance,
+      "vkGetPhysicalDeviceFormatProperties2");
+  if (!gst_vkGetPhysicalDeviceFormatProperties2)
+    gst_vkGetPhysicalDeviceFormatProperties2 =
+        gst_vulkan_instance_get_proc_address (physical_device->instance,
+        "vkGetPhysicalDeviceFormatProperties2KHR");
+#else
+  gpointer gst_vkGetPhysicalDeviceFormatProperties2 = NULL;
+#endif
+
+  gpu = gst_vulkan_physical_device_get_handle (physical_device);
 
   for (i = 0; i < G_N_ELEMENTS (vk_formats_map); i++) {
     guint64 feats_primary, feats_secondary = 0;
@@ -654,39 +644,44 @@ gst_vulkan_format_from_video_info_2 (GstVulkanDevice * device,
     if (vk_formats_map[i].format != GST_VIDEO_INFO_FORMAT (info))
       continue;
 
-    gst_vulkan_physical_device_get_format_properties (device->physical_device,
-        vk_formats_map[i].vkfrmt, &props);
-    feats_primary = (tiling == VK_IMAGE_TILING_LINEAR) ?
-        props.linear_tiling_feat : props.optimal_tiling_feat;
+    feats_primary = _get_feature_flags (gpu,
+        gst_vkGetPhysicalDeviceFormatProperties2, vk_formats_map[i].vkfrmt,
+        tiling);
 
     if (vk_formats_map[i].vkfrmt != vk_formats_map[i].vkfrmts[0]) {
-      gst_vulkan_physical_device_get_format_properties (device->physical_device,
-          vk_formats_map[i].vkfrmts[0], &props);
-      feats_secondary = (tiling == VK_IMAGE_TILING_LINEAR) ?
-          props.linear_tiling_feat : props.optimal_tiling_feat;
+      feats_secondary = _get_feature_flags (gpu,
+          gst_vkGetPhysicalDeviceFormatProperties2,
+          vk_formats_map[i].vkfrmts[0], tiling);
     }
 
     if (GST_VIDEO_INFO_IS_RGB (info)) {
-      usage = _get_usage (feats_primary);
-      if ((requested_usage & usage) == requested_usage) {
-        if (fmts)
-          fmts[0] = vk_formats_map[i].vkfrmt;
-        if (n_imgs)
-          *n_imgs = 1;
-        if (usage_ret)
-          *usage_ret = usage;
-        return TRUE;
+      if ((GST_VIDEO_INFO_COLORIMETRY (info).transfer ==
+              GST_VIDEO_TRANSFER_SRGB
+              || GST_VIDEO_INFO_COLORIMETRY (info).transfer ==
+              GST_VIDEO_TRANSFER_UNKNOWN)) {
+        usage = _get_usage (feats_primary);
+        if ((requested_usage & usage) == requested_usage) {
+          if (fmts)
+            fmts[0] = vk_formats_map[i].vkfrmt;
+          if (n_imgs)
+            *n_imgs = 1;
+          if (usage_ret)
+            *usage_ret = usage;
+          return TRUE;
+        }
       }
 
-      usage = _get_usage (feats_secondary);
-      if ((requested_usage & usage) == requested_usage) {
-        if (fmts)
-          fmts[0] = vk_formats_map[i].vkfrmts[0];
-        if (n_imgs)
-          *n_imgs = 1;
-        if (usage_ret)
-          *usage_ret = usage;
-        return TRUE;
+      if (GST_VIDEO_INFO_COLORIMETRY (info).transfer != GST_VIDEO_TRANSFER_SRGB) {
+        usage = _get_usage (feats_secondary);
+        if ((requested_usage & usage) == requested_usage) {
+          if (fmts)
+            fmts[0] = vk_formats_map[i].vkfrmts[0];
+          if (n_imgs)
+            *n_imgs = 1;
+          if (usage_ret)
+            *usage_ret = usage;
+          return TRUE;
+        }
       }
       return FALSE;
     } else {
