@@ -500,12 +500,13 @@ gst_base_auto_convert_add_element (GstBaseAutoConvert * self,
 
   g_assert (filter_info->subbin);
 
-  element = gst_object_ref (filter_info->subbin);
+
+  element = filter_info->subbin;
   GST_DEBUG_OBJECT (self, "Adding element %s to the baseautoconvert bin",
       filter_info->name);
 
   pads = internal_pads_new (self, GST_OBJECT_NAME (element));
-  g_hash_table_insert (self->elements, element, internal_pads_ref (pads));
+  g_hash_table_insert (self->elements, element, pads);
 
   gst_pad_set_chain_function (pads->sink,
       GST_DEBUG_FUNCPTR (gst_base_auto_convert_internal_sink_chain));
@@ -521,9 +522,7 @@ gst_base_auto_convert_add_element (GstBaseAutoConvert * self,
   gst_pad_set_query_function (pads->src,
       GST_DEBUG_FUNCPTR (gst_base_auto_convert_internal_src_query));
 
-  internal_pads_unref (pads);
-
-  return element;
+  return gst_object_ref (element);
 }
 
 static GstElement *
@@ -627,8 +626,7 @@ gst_base_auto_convert_activate_element (GstBaseAutoConvert * self,
   }
 
   if (!gst_element_sync_state_with_parent (element)) {
-    GST_WARNING_OBJECT (self, "Could not sync %" GST_PTR_FORMAT " state",
-        element);
+    GST_WARNING_OBJECT (self, "Could sync %" GST_PTR_FORMAT " state", element);
     goto error;
   }
 
@@ -871,6 +869,8 @@ gst_base_auto_convert_sink_setcaps (GstBaseAutoConvert * self, GstCaps * caps,
     if (gst_base_auto_convert_activate_element (self, element, caps)) {
       res = TRUE;
       break;
+    } else {
+      gst_object_unref (element);
     }
   }
 
@@ -1152,16 +1152,6 @@ gst_base_auto_convert_sink_query (GstPad * pad, GstObject * parent,
     return ret;
   }
 
-  /* Should not forward the allocation query downstream directly
-   * if no subelement is selected, otherwise it can influence
-   * the downstream allocation choices and upstream buffer usage.
-   */
-  if (GST_QUERY_TYPE (query) == GST_QUERY_ALLOCATION) {
-    GST_DEBUG_OBJECT (self,
-        "no subelement is selected yet, can't answer ALLOCATION query");
-    return FALSE;
-  }
-
 ignore_acceptcaps_failure:
 
   if (GST_QUERY_TYPE (query) == GST_QUERY_ACCEPT_CAPS) {
@@ -1211,7 +1201,8 @@ gst_base_auto_convert_getcaps (GstBaseAutoConvert * self, GstCaps * filter,
     other_caps = gst_pad_peer_query_caps (self->sinkpad, NULL);
 
   GST_DEBUG_OBJECT (self,
-      "Finding elements that can fit with caps %" GST_PTR_FORMAT, other_caps);
+      "Finding elements that can fit with src caps %" GST_PTR_FORMAT,
+      other_caps);
 
   if (other_caps && gst_caps_is_empty (other_caps)) {
     goto out;
@@ -1228,7 +1219,7 @@ gst_base_auto_convert_getcaps (GstBaseAutoConvert * self, GstCaps * filter,
       if (!filter_info_can_intersect (self, filter_info, dir, filter)) {
         GST_LOG_OBJECT (self,
             "Bin %s does not accept %s caps %" GST_PTR_FORMAT,
-            filter_info->name, dir == GST_PAD_SRC ? "src" : "sink", filter);
+            filter_info->name, dir == GST_PAD_SRC ? "src" : "sink", other_caps);
         continue;
       }
     }
