@@ -31,6 +31,7 @@
 #include <gst/pbutils/pbutils.h>
 #include <gst/codecparsers/gstmpegvideometa.h>
 
+#include "gstvideoparserselements.h"
 #include "gstmpegvideoparse.h"
 
 GST_DEBUG_CATEGORY (mpegv_parse_debug);
@@ -64,6 +65,9 @@ enum
 
 #define parent_class gst_mpegv_parse_parent_class
 G_DEFINE_TYPE (GstMpegvParse, gst_mpegv_parse, GST_TYPE_BASE_PARSE);
+GST_ELEMENT_REGISTER_DEFINE_WITH_CODE (mpegvideoparse, "mpegvideoparse",
+    GST_RANK_PRIMARY + 1, GST_TYPE_MPEGVIDEO_PARSE,
+    videoparsers_element_init (plugin));
 
 static gboolean gst_mpegv_parse_start (GstBaseParse * parse);
 static gboolean gst_mpegv_parse_stop (GstBaseParse * parse);
@@ -137,7 +141,7 @@ gst_mpegv_parse_class_init (GstMpegvParseClass * klass)
 
   g_object_class_install_property (gobject_class, PROP_DROP,
       g_param_spec_boolean ("drop", "drop",
-          "Drop data untill valid configuration data is received either "
+          "Drop data until valid configuration data is received either "
           "in the stream or through caps", DEFAULT_PROP_DROP,
           G_PARAM_CONSTRUCT | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
@@ -146,10 +150,8 @@ gst_mpegv_parse_class_init (GstMpegvParseClass * klass)
           "Split frame when encountering GOP", DEFAULT_PROP_GOP_SPLIT,
           G_PARAM_CONSTRUCT | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&src_template));
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&sink_template));
+  gst_element_class_add_static_pad_template (element_class, &src_template);
+  gst_element_class_add_static_pad_template (element_class, &sink_template);
 
   gst_element_class_set_static_metadata (element_class,
       "MPEG video elementary stream parser",
@@ -176,6 +178,7 @@ gst_mpegv_parse_init (GstMpegvParse * parse)
   parse->config_flags = FLAG_NONE;
 
   gst_base_parse_set_pts_interpolation (GST_BASE_PARSE (parse), FALSE);
+  gst_base_parse_set_infer_ts (GST_BASE_PARSE (parse), FALSE);
   GST_PAD_SET_ACCEPT_INTERSECT (GST_BASE_PARSE_SINK_PAD (parse));
   GST_PAD_SET_ACCEPT_TEMPLATE (GST_BASE_PARSE_SINK_PAD (parse));
 }
@@ -193,6 +196,7 @@ gst_mpegv_parse_reset_frame (GstMpegvParse * mpvparse)
   mpvparse->ext_count = 0;
   mpvparse->slice_count = 0;
   mpvparse->slice_offset = 0;
+  gst_video_clear_user_data (&mpvparse->user_data);
 }
 
 static void
@@ -387,17 +391,17 @@ picture_start_code_name (guint8 psc)
     const gchar *name;
   } psc_names[] = {
     {
-    0x00, "Picture Start"}, {
-    0xb0, "Reserved"}, {
-    0xb1, "Reserved"}, {
-    0xb2, "User Data Start"}, {
-    0xb3, "Sequence Header Start"}, {
-    0xb4, "Sequence Error"}, {
-    0xb5, "Extension Start"}, {
-    0xb6, "Reserved"}, {
-    0xb7, "Sequence End"}, {
-    0xb8, "Group Start"}, {
-    0xb9, "Program End"}
+        0x00, "Picture Start"}, {
+        0xb0, "Reserved"}, {
+        0xb1, "Reserved"}, {
+        0xb2, "User Data Start"}, {
+        0xb3, "Sequence Header Start"}, {
+        0xb4, "Sequence Error"}, {
+        0xb5, "Extension Start"}, {
+        0xb6, "Reserved"}, {
+        0xb7, "Sequence End"}, {
+        0xb8, "Group Start"}, {
+        0xb9, "Program End"}
   };
   if (psc < 0xB0 && psc > 0)
     return "Slice Start";
@@ -419,11 +423,11 @@ picture_type_name (guint8 pct)
     const gchar *name;
   } pct_names[] = {
     {
-    0, "Forbidden"}, {
-    1, "I Frame"}, {
-    2, "P Frame"}, {
-    3, "B Frame"}, {
-    4, "DC Intra Coded (Shall Not Be Used!)"}
+        0, "Forbidden"}, {
+        1, "I Frame"}, {
+        2, "P Frame"}, {
+        3, "B Frame"}, {
+        4, "DC Intra Coded (Shall Not Be Used!)"}
   };
 
   for (i = 0; i < G_N_ELEMENTS (pct_names); i++)
@@ -463,6 +467,22 @@ parse_packet_extension (GstMpegvParse * mpvparse, GstMapInfo * info, guint off)
   }
 }
 
+static void
+parse_user_data_packet (GstMpegvParse * mpvparse, const guint8 * data,
+    guint size)
+{
+  GstByteReader br;
+  GstVideoParseUtilsField field = GST_VIDEO_PARSE_UTILS_FIELD_1;
+  gst_byte_reader_init (&br, data, size);
+
+  if (mpvparse->picext.picture_structure ==
+      (guint8) GST_MPEG_VIDEO_PICTURE_STRUCTURE_BOTTOM_FIELD)
+    field = GST_VIDEO_PARSE_UTILS_FIELD_2;
+  gst_video_parse_user_data ((GstElement *) mpvparse, &mpvparse->user_data, &br,
+      field, ITU_T_T35_MANUFACTURER_US_ATSC);
+
+}
+
 /* caller guarantees at least start code in @buf at @off ( - 4)*/
 /* for off == 4 initial code; returns TRUE if code starts a frame
  * otherwise returns TRUE if code terminates preceding frame */
@@ -517,6 +537,18 @@ gst_mpegv_parse_process_sc (GstMpegvParse * mpvparse,
       }
       checkconfig = FALSE;
       break;
+    case GST_MPEG_VIDEO_PACKET_USER_DATA:
+      GST_LOG_OBJECT (mpvparse, "USER_DATA packet of %d bytes", packet->size);
+
+      if (packet->size < 0) {
+        GST_LOG_OBJECT (mpvparse, "no size yet, need more data");
+        *need_more = TRUE;
+        return FALSE;
+      }
+
+      parse_user_data_packet (mpvparse, info->data + off, packet->size);
+      checkconfig = FALSE;
+      break;
     default:
       if (GST_MPEG_VIDEO_PACKET_IS_SLICE (packet->type)) {
         mpvparse->slice_count++;
@@ -815,6 +847,7 @@ gst_mpegv_parse_update_src_caps (GstMpegvParse * mpvparse)
   }
 
   if (mpvparse->config_flags & FLAG_SEQUENCE_EXT) {
+    guint8 escape = mpvparse->sequenceext.profile_level_escape_bit;
     const guint profile_c = mpvparse->sequenceext.profile;
     const guint level_c = mpvparse->sequenceext.level;
     const gchar *profile = NULL, *level = NULL;
@@ -824,20 +857,14 @@ gst_mpegv_parse_update_src_caps (GstMpegvParse * mpvparse)
      * 4:2:2 and Multi-view have profile = 0, with the escape bit set to 1
      */
     const gchar *const profiles[] =
-        { "high", "spatial", "snr", "main", "simple" };
+        { "4:2:2", "high", "spatial", "snr", "main", "simple" };
     /*
      * Level indication - 4 => High, 6 => High-1440, 8 => Main, 10 => Low,
      *                    except in the case of profile = 0
      */
     const gchar *const levels[] = { "high", "high-1440", "main", "low" };
 
-    if (profile_c > 0 && profile_c < 6)
-      profile = profiles[profile_c - 1];
-
-    if ((level_c > 3) && (level_c < 11) && (level_c % 2 == 0))
-      level = levels[(level_c >> 1) - 2];
-
-    if (profile_c == 8) {
+    if (escape) {
       /* Non-hierarchical profile */
       switch (level_c) {
         case 2:
@@ -863,9 +890,14 @@ gst_mpegv_parse_update_src_caps (GstMpegvParse * mpvparse)
         default:
           break;
       }
-    }
 
-    /* FIXME does it make sense to expose profile/level in the caps ? */
+    } else {
+      if (profile_c < 6)
+        profile = profiles[profile_c];
+
+      if ((level_c > 3) && (level_c < 11) && (level_c % 2 == 0))
+        level = levels[(level_c >> 1) - 2];
+    }
 
     GST_DEBUG_OBJECT (mpvparse, "profile:'%s' level:'%s'", profile, level);
 
@@ -879,9 +911,10 @@ gst_mpegv_parse_update_src_caps (GstMpegvParse * mpvparse)
     else
       GST_DEBUG_OBJECT (mpvparse, "Invalid level - %u", level_c);
 
-    gst_caps_set_simple (caps, "interlace-mode",
-        G_TYPE_STRING,
-        (mpvparse->sequenceext.progressive ? "progressive" : "mixed"), NULL);
+    if (!s || !gst_structure_has_field (s, "interlace-mode"))
+      gst_caps_set_simple (caps, "interlace-mode",
+          G_TYPE_STRING,
+          (mpvparse->sequenceext.progressive ? "progressive" : "mixed"), NULL);
   }
 
   gst_pad_set_caps (GST_BASE_PARSE_SRC_PAD (mpvparse), caps);
@@ -898,7 +931,7 @@ gst_mpegv_parse_parse_frame (GstBaseParse * parse, GstBaseParseFrame * frame)
 
   if (G_UNLIKELY (mpvparse->pichdr.pic_type == GST_MPEG_VIDEO_PICTURE_TYPE_I))
     GST_BUFFER_FLAG_UNSET (buffer, GST_BUFFER_FLAG_DELTA_UNIT);
-  else
+  else if (mpvparse->pichdr.pic_type != 0)
     GST_BUFFER_FLAG_SET (buffer, GST_BUFFER_FLAG_DELTA_UNIT);
 
   /* maybe only sequence in this buffer, though not recommended,
@@ -940,6 +973,7 @@ gst_mpegv_parse_pre_push_frame (GstBaseParse * parse, GstBaseParseFrame * frame)
   GstMpegVideoPictureHdr *pic_hdr = NULL;
   GstMpegVideoPictureExt *pic_ext = NULL;
   GstMpegVideoQuantMatrixExt *quant_ext = NULL;
+  GstBuffer *parse_buffer = NULL;
 
   /* tag sending done late enough in hook to ensure pending events
    * have already been sent */
@@ -948,8 +982,18 @@ gst_mpegv_parse_pre_push_frame (GstBaseParse * parse, GstBaseParseFrame * frame)
     GstCaps *caps;
 
     /* codec tag */
-    taglist = gst_tag_list_new_empty ();
     caps = gst_pad_get_current_caps (GST_BASE_PARSE_SRC_PAD (parse));
+    if (G_UNLIKELY (caps == NULL)) {
+      if (GST_PAD_IS_FLUSHING (GST_BASE_PARSE_SRC_PAD (parse))) {
+        GST_INFO_OBJECT (parse, "Src pad is flushing");
+        return GST_FLOW_FLUSHING;
+      } else {
+        GST_INFO_OBJECT (parse, "Src pad is not negotiated!");
+        return GST_FLOW_NOT_NEGOTIATED;
+      }
+    }
+
+    taglist = gst_tag_list_new_empty ();
     gst_pb_utils_add_codec_description_to_tag_list (taglist,
         GST_TAG_VIDEO_CODEC, caps);
     gst_caps_unref (caps);
@@ -960,9 +1004,6 @@ gst_mpegv_parse_pre_push_frame (GstBaseParse * parse, GstBaseParseFrame * frame)
     mpvparse->send_codec_tag = FALSE;
   }
 
-  /* usual clipping applies */
-  frame->flags |= GST_BASE_PARSE_FRAME_FLAG_CLIP;
-
   if (mpvparse->send_mpeg_meta) {
     GstBuffer *buf;
 
@@ -994,6 +1035,22 @@ gst_mpegv_parse_pre_push_frame (GstBaseParse * parse, GstBaseParseFrame * frame)
     meta->num_slices = mpvparse->slice_count;
     meta->slice_offset = mpvparse->slice_offset;
   }
+
+  if (frame->out_buffer) {
+    parse_buffer = frame->out_buffer =
+        gst_buffer_make_writable (frame->out_buffer);
+  } else {
+    parse_buffer = frame->buffer = gst_buffer_make_writable (frame->buffer);
+  }
+
+  if (pic_ext && !pic_ext->progressive_frame) {
+    GST_BUFFER_FLAG_SET (parse_buffer, GST_VIDEO_BUFFER_FLAG_INTERLACED);
+    if (pic_ext->top_field_first)
+      GST_BUFFER_FLAG_SET (parse_buffer, GST_VIDEO_BUFFER_FLAG_TFF);
+  }
+  gst_video_push_user_data ((GstElement *) mpvparse, &mpvparse->user_data,
+      parse_buffer);
+
   return GST_FLOW_OK;
 }
 
@@ -1015,7 +1072,7 @@ gst_mpegv_parse_set_caps (GstBaseParse * parse, GstCaps * caps)
     gst_buffer_map (buf, &map, GST_MAP_READ);
     /* best possible parse attempt,
      * src caps are based on sink caps so it will end up in there
-     * whether sucessful or not */
+     * whether successful or not */
     mpvparse->seq_offset = 4;
     gst_mpegv_parse_process_config (mpvparse, &map, gst_buffer_get_size (buf));
     gst_buffer_unmap (buf, &map);
