@@ -32,7 +32,6 @@
 #include <gst/vulkan/vulkan.h>
 #endif
 #include <gst/codecparsers/gsth264parser.h>
-#include <gst/codecparsers/gstav1parser.h>
 
 G_BEGIN_DECLS
 
@@ -57,7 +56,7 @@ struct _GstVtdec
   GstVideoTextureCache *texture_cache;
   GstGLContextHelper *ctxh;
 
-  GstVecDeque *reorder_queue;
+  GstQueueArray *reorder_queue;
   gint dbp_size;
   GMutex queue_mutex;
   GCond queue_cond;
@@ -73,12 +72,6 @@ struct _GstVtdec
 #endif
 
   gboolean require_hardware;
-
-  gboolean av1_needs_sequence_header;  /* TRUE if we need to wait for sequence header OBU before creating session */
-  GstBuffer *av1_sequence_header_obu;  /* Store the sequence header OBU for format description */
-
-  guint8* vp9_vpcc;
-  gsize   vp9_vpcc_size;
 };
 
 struct _GstVtdecClass
