@@ -173,42 +173,28 @@ gst_v4l2_codec_allocator_prepare (GstV4l2CodecAllocator * self)
 {
   GstV4l2Decoder *decoder = self->decoder;
   GstPadDirection direction = self->direction;
-  GstV4l2CodecBuffer *buf;
+  gint ret;
   guint i;
 
-  GST_DEBUG_OBJECT (self, "Try to create %d buffers", self->pool_size);
-
-  /* Allocate buffers one by one to avoid fragmentation issue and
-   * use the possible holes in v4l2 queue array */
-  for (i = 0; i < self->pool_size; i++) {
-    GstV4l2CodecBuffer *buf;
-    gint index = gst_v4l2_decoder_create_buffers (decoder, direction, 1);
-    if (index < 0) {
+  ret = gst_v4l2_decoder_request_buffers (decoder, direction, self->pool_size);
+  if (ret < self->pool_size) {
+    if (ret >= 0)
       GST_ERROR_OBJECT (self,
           "%i buffer was needed, but only %i could be allocated",
-          self->pool_size, i);
-      goto failed;
-    }
+          self->pool_size, ret);
+    goto failed;
+  }
 
-    buf =
-        gst_v4l2_codec_buffer_new (GST_ALLOCATOR (self), decoder, direction,
-        index);
+  for (i = 0; i < self->pool_size; i++) {
+    GstV4l2CodecBuffer *buf = gst_v4l2_codec_buffer_new (GST_ALLOCATOR (self),
+        decoder, direction, i);
     g_queue_push_tail (&self->pool, buf);
   }
 
   return TRUE;
 
 failed:
-  /* If buffer allocation failed remove them all either by
-   * calling delete_buffers IOCTL if the driver support it,
-   * either by using legacy request_buf IOCTL with 0 has parameter */
-  if (gst_v4l2_decoder_has_remove_bufs (decoder)) {
-    while (i-- && (buf = g_queue_pop_tail (&self->pool))) {
-      gst_v4l2_decoder_remove_buffers (decoder, direction, buf->index, 1);
-    }
-  } else {
-    gst_v4l2_decoder_request_buffers (decoder, direction, 0);
-  }
+  gst_v4l2_decoder_request_buffers (decoder, direction, 0);
   return FALSE;
 }
 
@@ -222,16 +208,10 @@ static void
 gst_v4l2_codec_allocator_dispose (GObject * object)
 {
   GstV4l2CodecAllocator *self = GST_V4L2_CODEC_ALLOCATOR (object);
-  GstV4l2Decoder *decoder = self->decoder;
-  GstPadDirection direction = self->direction;
   GstV4l2CodecBuffer *buf;
 
-  while ((buf = g_queue_pop_head (&self->pool))) {
-    if (gst_v4l2_decoder_has_remove_bufs (decoder)) {
-      gst_v4l2_decoder_remove_buffers (decoder, direction, buf->index, 1);
-    }
+  while ((buf = g_queue_pop_head (&self->pool)))
     gst_v4l2_codec_buffer_free (buf);
-  }
 
   if (self->decoder) {
     gst_v4l2_codec_allocator_detach (self);
@@ -365,21 +345,10 @@ gst_v4l2_codec_allocator_get_pool_size (GstV4l2CodecAllocator * self)
 void
 gst_v4l2_codec_allocator_detach (GstV4l2CodecAllocator * self)
 {
-  GstV4l2Decoder *decoder = self->decoder;
-
   GST_OBJECT_LOCK (self);
   if (!self->detached) {
     self->detached = TRUE;
-    if (!gst_v4l2_decoder_has_remove_bufs (decoder)) {
-      gst_v4l2_decoder_request_buffers (self->decoder, self->direction, 0);
-    } else {
-      GstV4l2CodecBuffer *buf;
-
-      while ((buf = g_queue_pop_tail (&self->pool)))
-        gst_v4l2_decoder_remove_buffers (self->decoder, self->direction,
-            buf->index, 1);
-
-    }
+    gst_v4l2_decoder_request_buffers (self->decoder, self->direction, 0);
   }
   GST_OBJECT_UNLOCK (self);
 }
